﻿using System;
using Force.DeepCloner;
using HarmonyLib;
using Microsoft.Xna.Framework;
using StardewModdingAPI;
using StardewModdingAPI.Events;
using StardewModdingAPI.Utilities;
using StardewValley;
using StardewValley.GameData.BigCraftables;
using StardewValley.GameData.Machines;
using StardewValley.GameData.Objects;
using StardewValley.Inventories;
using StardewValley.Objects;

namespace YourProjectName
{
    /// <summary>The mod entry point.</summary>
    internal sealed class ModEntry : Mod
    {
        struct DCpair
        {
            public Chest donor;
            public Chest rec;
            public StardewValley.Object donorM;
            public StardewValley.Object recM;
            public List<Chest> Signals;
        }

        static Dictionary<Chest, DCpair> DCpairs = new Dictionary<Chest, DCpair>();
        static Dictionary<StardewValley.Buildings.Building, Chest> Bpairs = new Dictionary<StardewValley.Buildings.Building, Chest>();

        BigCraftableData dcup = new BigCraftableData();
        BigCraftableData dcdown = new BigCraftableData();
        BigCraftableData dcleft = new BigCraftableData();
        BigCraftableData dcright = new BigCraftableData();
        BigCraftableData signalC = new BigCraftableData();

        ObjectData signal = new ObjectData();

        public override void Entry(IModHelper helper)
        {
            helper.Events.Content.AssetRequested += this.OnAssetRequested;
            helper.Events.GameLoop.DayStarted += this.OnDayStarted;
            helper.Events.GameLoop.TimeChanged += this.OnTimeChanged;

            var harmony = new Harmony(this.ModManifest.UniqueID);

            harmony.Patch(
            original: AccessTools.Method(typeof(StardewValley.Object), nameof(StardewValley.Object.placementAction)),
            postfix: new HarmonyMethod(typeof(ModEntry), nameof(GetplacementAction2))
            );

            harmony.Patch(
            original: AccessTools.Method(typeof(StardewValley.Object), nameof(StardewValley.Object.performRemoveAction)),
            postfix: new HarmonyMethod(typeof(ModEntry), nameof(GetperformRemoveAction))
            );
        }

        private void OnGameLaunched(object? sender, GameLaunchedEventArgs e)
        {

            Game1.bigCraftableData["130"].DeepCloneTo(dcup);
            dcup.Description = "Its a Direction chest! It moves items up.";
            dcup.Name = "Direction Chest (Up)";
            dcup.DisplayName = "Direction Chest (Up)";
            dcup.Texture = "Tilesheets/DirectionChests";
            dcup.SpriteIndex = 0;



            Game1.bigCraftableData["130"].DeepCloneTo(dcdown);
            dcdown.Description = "Its a Direction chest! It moves items down.";
            dcdown.Name = "Direction Chest (Down)";
            dcdown.DisplayName = "Direction Chest (Down)";
            dcdown.Texture = "Tilesheets/DirectionChests";
            dcdown.SpriteIndex = 7;


            Game1.bigCraftableData["130"].DeepCloneTo(dcleft);
            dcleft.Description = "Its a Direction chest! It moves items left.";
            dcleft.Name = "Direction Chest (Left)";
            dcleft.DisplayName = "Direction Chest (Left)";
            dcleft.Texture = "Tilesheets/DirectionChests";
            dcleft.SpriteIndex = 21;

            Game1.bigCraftableData["130"].DeepCloneTo(dcright);
            dcright.Description = "Its a Direction chest! It moves items right.";
            dcright.Name = "Direction Chest (Right)";
            dcright.DisplayName = "Direction Chest (Right)";
            dcright.Texture = "Tilesheets/DirectionChests";
            dcright.SpriteIndex = 14;

            Game1.bigCraftableData["130"].DeepCloneTo(signalC);
            signalC.Description = "Its a Signal chest! It stops direction chests if it doesnt have a signal.";
            signalC.Name = "Signal chest";
            signalC.DisplayName = "Signal chest";

            Game1.objectData["64"].DeepCloneTo(signal);
            signal.Description = "A Signal";
            signal.Name = "Signal";
            signal.DisplayName = "Signal";
            signal.Price = 0;


        }

        private void OnAssetRequested(object sender, AssetRequestedEventArgs e)
        {
            if (e.NameWithoutLocale.IsEquivalentTo("Data/BigCraftables"))
            {
                e.Edit(asset =>
                {
                    var data = asset.AsDictionary<string, BigCraftableData>().Data;

                    data.Add("DirectionChestUp", dcup);
                    data.Add("DirectionChestDown", dcdown);
                    data.Add("DirectionChestLeft", dcleft);
                    data.Add("DirectionChestRight", dcright);
                    data.Add("SignalChest", signalC);
                });
            }
            if (e.NameWithoutLocale.IsEquivalentTo("Data/CraftingRecipes"))
            {
                e.Edit(asset =>
                {
                    var data = asset.AsDictionary<string, string>().Data;

                    data.Add("DirectionChestUp", "388 50/Home/DirectionChestUp/true/default/");
                    data.Add("DirectionChestDown", "388 50/Home/DirectionChestDown/true/default/");
                    data.Add("DirectionChestLeft", "388 50/Home/DirectionChestLeft/true/default/");
                    data.Add("DirectionChestRight", "388 50/Home/DirectionChestRight/true/default/");
                    data.Add("SignalChest", "388 50/Home/SignalChest/true/default/");
                    data.Add("Signal", "388 0/Home/SIGNAL/false/default/");
                });
            }
            if (e.NameWithoutLocale.IsEquivalentTo("Data/Objects"))
            {
                e.Edit(asset =>
                {

                    var data = asset.AsDictionary<string, ObjectData>().Data;
                    data.Add("SIGNAL", signal);
                });
            }
        }

        private void OnDayStarted(object? sender, DayStartedEventArgs e)
        {
            InitiateDirectionChests();
            InitiateBuildingChests();
        }

        private async void OnTimeChanged(object? sender, TimeChangedEventArgs e)
        {

            foreach (KeyValuePair<Chest, DCpair> pair in DCpairs)
            {

                if (pair.Value.rec != null && pair.Value.donor != null)
                {
                    transfer1(pair.Key, pair.Value.donor, pair.Value.rec);
                }
                else if (pair.Value.recM != null && pair.Value.donorM != null)
                {
                    transfer2(pair.Key, pair.Value.donorM, pair.Value.recM);
                }
                else if (pair.Value.recM != null && pair.Value.donor != null)
                {
                    transfer3(pair.Key, pair.Value.donor, pair.Value.recM);
                }
                else if (pair.Value.rec != null && pair.Value.donorM != null)
                {
                    transfer4(pair.Key, pair.Value.donorM, pair.Value.rec);
                }

            }
        }




        static void InitiateDirectionChests()
        {
            DCpairs = new Dictionary<Chest, DCpair>();


            int x = 0;
            int y = 0;

            foreach (GameLocation loc in Game1.locations)  // for every game location
            {
                for (y = 0; y < 100; y++)
                {
                    for (x = 0; x < 100; x++)
                    {
                        if (loc.hasTileAt(x, y, "Back"))
                        {
                            if (loc.getObjectAtTile(x, y) != null && loc.getObjectAtTile(x, y).ItemId.Contains("DirectionChest")) //if the object in the tile is a direction chest
                            {
                                Chest DC = (Chest)loc.getObjectAtTile(x, y);
                                Vector2 upv = new Vector2(x, y + 1);
                                Vector2 downv = new Vector2(x, y - 1);
                                Vector2 leftv = new Vector2(x - 1, y);
                                Vector2 rightv = new Vector2(x + 1, y);

                                // Does the up and down tiles have an object? Or do the left and right tiles have an object?
                                if ((!loc.CanItemBePlacedHere(upv) && !loc.CanItemBePlacedHere(downv)) || (!loc.CanItemBePlacedHere(leftv) && !loc.CanItemBePlacedHere(rightv)))
                                {
                                    StardewValley.Object donor = null;
                                    StardewValley.Object rec = null;

                                    switch (DC.ItemId)  //find the first adjacent chests.
                                    {
                                        case "DirectionChestLeft":
                                            donor = DC.Location.getObjectAtTile((int)x + 1, (int)y);
                                            rec = DC.Location.getObjectAtTile((int)x - 1, (int)y);
                                            break;
                                        case "DirectionChestRight":
                                            donor = DC.Location.getObjectAtTile((int)x - 1, (int)y);
                                            rec = DC.Location.getObjectAtTile((int)x + 1, (int)y);
                                            break;
                                        case "DirectionChestUp":
                                            donor = DC.Location.getObjectAtTile((int)x, (int)y + 1);
                                            rec = DC.Location.getObjectAtTile((int)x, (int)y - 1);
                                            break;
                                        case "DirectionChestDown":
                                            donor = DC.Location.getObjectAtTile((int)x, (int)y - 1);
                                            rec = DC.Location.getObjectAtTile((int)x, (int)y + 1);
                                            break;

                                    }

                                    if (donor != null && rec != null && donor.ItemId.Contains("DirectionChest") && rec.ItemId.Contains("DirectionChest")) // if both donor and rec are direction chests, then the given chest is on a path and is not counted.
                                    {
                                        continue;
                                    }
                                    else
                                    {

                                        startTransfer(DC, donor, rec); //else, try to start a transfer.
                                    }

                                }

                            }

                        }
                    }

                }
            }
        }
        static void addStack(String id, Chest inv, int amount)  // adds to a stack
        {
            if (inv.GetActualCapacity() <= inv.Items.Count) // if the count is equal to or larger than count
            {
                //Console.WriteLine("false return" + inv.ItemId + inv.TileLocation);
                return;
            }
            if (inv.Items.ContainsId(id))  //if it contains the id
            {
                bool didAdd = false;
                foreach (Item i in inv.Items.GetById(id))
                {
                    if (i.getRemainingStackSpace() < amount)  //if not enough space
                    {
                        //Console.WriteLine("Not enough space" + inv.ItemId + inv.TileLocation);
                        continue;
                    }
                    else
                    {
                        didAdd = true;  //then add it
                        i.Stack += amount;
                        break;
                    }
                }
                if (!didAdd)
                {


                    inv.addItem(new StardewValley.Object(id, amount));
                }
            }
            else
            {

                inv.addItem(new StardewValley.Object(id, amount));
            }
        }
        static public void startTransfer(Chest DC, StardewValley.Object donor, StardewValley.Object rec)
        {
            int uhoh = 0;
            float x = DC.TileLocation.X;
            float y = DC.TileLocation.Y;

            if (rec == null || donor == null)
            {
                return;
            }


            try
            {
                while (donor.ItemId.Contains("DirectionChest"))  //find the parent donor obj if it exists
                {
                    float x1 = donor.TileLocation.X;
                    float y1 = donor.tileLocation.Y;


                    switch (donor.ItemId)
                    {
                        case "DirectionChestLeft":
                            if (donor.Location.CanItemBePlacedHere(new Vector2((int)x1 + 1, (int)y1)))
                            {
                                return;  //return if oob
                            }

                            donor = donor.Location.getObjectAtTile((int)x1 + 1, (int)y1);
                            break;
                        case "DirectionChestRight":
                            if (donor.Location.CanItemBePlacedHere(new Vector2((int)x1 - 1, (int)y1)))
                            {
                                return;
                            }
                            donor = donor.Location.getObjectAtTile((int)x1 - 1, (int)y1);
                            break;
                        case "DirectionChestUp":
                            if (donor.Location.CanItemBePlacedHere(new Vector2((int)x1, (int)y1 + 1)))
                            {
                                return;
                            }
                            donor = donor.Location.getObjectAtTile((int)x1, (int)y1 + 1);

                            break;
                        case "DirectionChestDown":
                            if (donor.Location.CanItemBePlacedHere(new Vector2((int)x1, (int)y1 - 1)))
                            {
                                return;
                            }
                            donor = donor.Location.getObjectAtTile((int)x1, (int)y1 - 1);
                            break;

                    }
                    if (++uhoh > 100)
                    {
                        return;
                    }
                }
            }
            catch (NullReferenceException one)
            {
                donor = new Chest();
                return;
            }




            uhoh = 0;

            try
            {
                while (rec.ItemId.Contains("DirectionChest"))  // find parent rec if it exists
                {
                    float x1 = rec.TileLocation.X;
                    float y1 = rec.tileLocation.Y;

                    switch (rec.ItemId)
                    {
                        case "DirectionChestLeft":
                            if (rec.Location.CanItemBePlacedHere(new Vector2((int)x1 - 1, (int)y1)))
                            {
                                return;
                            }
                            rec = rec.Location.getObjectAtTile((int)x1 - 1, (int)y1);
                            break;
                        case "DirectionChestRight":
                            if (rec.Location.CanItemBePlacedHere(new Vector2((int)x1 + 1, (int)y1)))
                            {
                                return;
                            }
                            rec = rec.Location.getObjectAtTile((int)x1 + 1, (int)y1);
                            break;
                        case "DirectionChestUp":
                            if (rec.Location.CanItemBePlacedHere(new Vector2((int)x1, (int)y1 - 1)))
                            {
                                return;
                            }
                            rec = rec.Location.getObjectAtTile((int)x1, (int)y1 - 1);
                            break;
                        case "DirectionChestDown":
                            if (rec.Location.CanItemBePlacedHere(new Vector2((int)x1, (int)y1 + 1)))
                            {
                                return;
                            }
                            rec = rec.Location.getObjectAtTile((int)x1, (int)y1 + 1);
                            break;

                    }

                    if (++uhoh > 100)
                    {
                        return;
                    }
                }
            }
            catch (NullReferenceException two)
            {
                rec = new Chest();
                return;
            }


            if (rec == null || donor == null)
            {
                return;
            }



            if (rec.ItemId == "130" && donor.ItemId == "130")
            {
                DCpair d = new DCpair();
                d.rec = (Chest)rec;
                d.donor = (Chest)donor;
                DCpairs.Add(DC, d);

                transfer1(DC, ((Chest)donor), ((Chest)rec)); // transfer items
            }
            else if (rec.GetMachineData() != null && donor.GetMachineData() != null)
            {

                DCpair d = new DCpair();
                d.recM = rec;
                d.donorM = donor;
                DCpairs.Add(DC, d);

                transfer2(DC, donor, rec);
            }
            else if (rec.GetMachineData() != null && donor.ItemId == "130")
            {
                DCpair d = new DCpair();
                d.recM = rec;
                d.donor = (Chest)donor;
                DCpairs.Add(DC, d);

                transfer3(DC, ((Chest)donor), (rec));
            }
            else if (rec.ItemId == "130" && donor.GetMachineData() != null)
            {

                DCpair d = new DCpair();
                d.rec = (Chest)rec;
                d.donorM = donor;
                DCpairs.Add(DC, d);

                transfer4(DC, donor, ((Chest)rec));
            }

        }

        static void InitiateBuildingChests()
        {
            Bpairs = new Dictionary<StardewValley.Buildings.Building, Chest>();

            foreach (GameLocation loc in Game1.locations)  // for every game location
            {
                Chest B;

                foreach (StardewValley.Buildings.Building b in loc.buildings) //search the surrounding tiles for a chest
                {
                    for (int x = b.tileX.Value; x < b.tilesWide.Value + b.tileX.Value; x++)
                    {

                        if (loc.isObjectAtTile(x, b.tileY.Value - 1) && loc.getObjectAtTile(x, b.tileY.Value - 1).ItemId == "130")
                        {
                            B = (Chest)loc.getObjectAtTile(x, b.tileY.Value - 1);
                            Bpairs.Add(b, B);
                            transferB(b, B);
                            return;
                        }
                        if (loc.isObjectAtTile(x, b.tileY.Value + b.tilesHigh.Value) && loc.getObjectAtTile(x, b.tileY.Value + b.tilesHigh.Value).ItemId == ("130"))
                        {
                            B = (Chest)loc.getObjectAtTile(x, b.tileY.Value + b.tilesHigh.Value);
                            Bpairs.Add(b, B);
                            transferB(b, B);
                            return;
                        }
                    }
                    for (int y = b.tileY.Value; y < b.tileY.Value + b.tilesHigh.Value; y++)
                    {

                        if (loc.isObjectAtTile(b.tileX.Value - 1, y) && loc.getObjectAtTile(b.tileX.Value - 1, y).ItemId == "130")
                        {
                            B = (Chest)loc.getObjectAtTile(b.tileX.Value - 1, y);
                            Bpairs.Add(b, B);
                            transferB(b, B);
                            return;
                        }
                        if (loc.isObjectAtTile(b.tilesWide.Value + b.tileX.Value, y) && loc.getObjectAtTile(b.tilesWide.Value + b.tileX.Value, y).ItemId == ("130"))
                        {
                            B = (Chest)loc.getObjectAtTile(b.tilesWide.Value + b.tileX.Value, y);
                            Bpairs.Add(b, B);
                            transferB(b, B);
                            return;
                        }
                    }
                }
            }
        }
        static bool checkForSignal(Chest DC)
        {
            if (DCpairs[DC].Signals.Count == 0)
            {
                return true;
            }



            foreach (Chest signal in DCpairs[DC].Signals)
            {
                bool hasSignal = false;

                //Console.WriteLine(signal.TileLocation);
                foreach (Item i in signal.Items)
                {
                    //Console.WriteLine("Looking for signal at "+i.itemId);
                    if (i.ItemId == "SIGNAL")
                    {
                        hasSignal = true;
                    }
                }
                if (!hasSignal)
                {
                    return false;
                }
            }
            return true;
        }
        static void transfer1(Chest DC, Chest d, Chest r) //object transfer based on 2 chests
        {
            Console.WriteLine("Transfer 1" + DC.TileLocation);

            if (checkForSignal(DC) == false)  // if there is a signal chest with no signals, return.
            {
                Console.WriteLine("nosignal");
                return;
            }
            Inventory inv = new Inventory();
            foreach (StardewValley.Object i in d.GetItemsForPlayer())
            {
                if (i != null)
                {
                    inv.Add(i);
                    //Console.WriteLine("Item i: " + i.Name + d.TileLocation);
                }
            }



            foreach (StardewValley.Item obj in inv)
            {

                foreach (StardewValley.Item item in DC.Items)
                {

                    if (obj != null && item != null && r.Items.Count < r.GetActualCapacity() && (item.ItemId == obj.ItemId || DC.Items.ContainsId("SIGNAL")))
                    {
                        if (obj as ColoredObject != null)
                        {

                            Farmer f = new Farmer();
                            d.GetItemsForPlayer().Remove(obj);

                            addStack(obj.ItemId, r, obj.Stack);

                        }
                        else
                        {


                            Farmer f = new Farmer();
                            d.GetItemsForPlayer().Remove(obj);

                            addStack(obj.ItemId, r, obj.Stack);

                        }


                    }
                }


            }
        }

        static void transfer2(Chest DC, StardewValley.Object d, StardewValley.Object r)
        {

            if (checkForSignal(DC) == false)
            {
                return;
            }

            Farmer f = new Farmer();
            f.Items.Add(d.heldObject.Value);
            if (d.heldObject.Value != null && d.MinutesUntilReady == 0 && r.MinutesUntilReady == 0 && d.readyForHarvest.Value && !r.readyForHarvest.Value && r.heldObject.Value == null)
            {
                StardewValley.Object o = new StardewValley.Object();
                d.heldObject.Value.DeepCloneTo(o);
                d.heldObject.Value = null;
                if (!MachineDataUtility.HasAdditionalRequirements(f.Items, r.GetMachineData().AdditionalConsumedItems, out var failedRequirement))
                {
                    return;  // if the additional requirements for the machine recipe is not fuffilled
                }
                MachineOutputRule output = null;
                MachineOutputTriggerRule outputrule = null;
                MachineOutputRule ignorecount = null;
                MachineOutputTriggerRule ignoreoutputrule = null;

                //here is my little if statement pit. its better than having it in one big line so it exits.
                //blah blah blah, check if the machine output rule for the item exists

                if (!MachineDataUtility.TryGetMachineOutputRule(r, r.GetMachineData(), MachineOutputTrigger.None, o, f, DC.Location, out output, out outputrule, out ignorecount, out ignoreoutputrule))
                {
                    if (!MachineDataUtility.TryGetMachineOutputRule(r, r.GetMachineData(), MachineOutputTrigger.DayUpdate, o, f, DC.Location, out output, out outputrule, out ignorecount, out ignoreoutputrule))
                    {
                        if (!MachineDataUtility.TryGetMachineOutputRule(r, r.GetMachineData(), MachineOutputTrigger.ItemPlacedInMachine, o, f, DC.Location, out output, out outputrule, out ignorecount, out ignoreoutputrule))
                        {
                            if (!MachineDataUtility.TryGetMachineOutputRule(r, r.GetMachineData(), MachineOutputTrigger.MachinePutDown, o, f, DC.Location, out output, out outputrule, out ignorecount, out ignoreoutputrule))
                            {
                                if (!MachineDataUtility.TryGetMachineOutputRule(r, r.GetMachineData(), MachineOutputTrigger.OutputCollected, o, f, DC.Location, out output, out outputrule, out ignorecount, out ignoreoutputrule))
                                {

                                    return;
                                }
                            }
                        }
                    }

                }
                if (o.Stack < output.Triggers[0].RequiredCount)
                {
                    return;
                }
                r.PlaceInMachine(r.GetMachineData(), o, false, f);
            }



        }

        static void transfer3(Chest DC, Chest d, StardewValley.Object r)
        {

            if (checkForSignal(DC) == false)
            {
                return;
            }





            try
            {
                Farmer f = new Farmer();
                f.Items.AddRange(d.Items);
                foreach (StardewValley.Item o in d.Items)
                {
                    foreach (StardewValley.Item DCo in DC.Items) // for each object in d that matches the object in DC chest, unless its a signal then we ignore this.
                    {

                        if (o == null || DCo == null || r.heldObject == null)
                        {
                            continue;
                        }


                        if ((o.ItemId == DCo.ItemId || DC.Items.ContainsId("SIGNAL")) && !r.readyForHarvest.Value && r.MinutesUntilReady == 0 && r.heldObject.Value == null)
                        {
                            MachineOutputRule output = null;
                            MachineOutputTriggerRule outputrule = null;
                            MachineOutputRule ignorecount = null;
                            MachineOutputTriggerRule ignoreoutputrule = null;  //blah blah blah, check if the machine output rule for the item exists
                            if (!MachineDataUtility.TryGetMachineOutputRule(r, r.GetMachineData(), MachineOutputTrigger.None, o, f, DC.Location, out output, out outputrule, out ignorecount, out ignoreoutputrule))
                            {
                                if (!MachineDataUtility.TryGetMachineOutputRule(r, r.GetMachineData(), MachineOutputTrigger.DayUpdate, o, f, DC.Location, out output, out outputrule, out ignorecount, out ignoreoutputrule))
                                {
                                    if (!MachineDataUtility.TryGetMachineOutputRule(r, r.GetMachineData(), MachineOutputTrigger.ItemPlacedInMachine, o, f, DC.Location, out output, out outputrule, out ignorecount, out ignoreoutputrule))
                                    {
                                        if (!MachineDataUtility.TryGetMachineOutputRule(r, r.GetMachineData(), MachineOutputTrigger.MachinePutDown, o, f, DC.Location, out output, out outputrule, out ignorecount, out ignoreoutputrule))
                                        {
                                            if (!MachineDataUtility.TryGetMachineOutputRule(r, r.GetMachineData(), MachineOutputTrigger.OutputCollected, o, f, DC.Location, out output, out outputrule, out ignorecount, out ignoreoutputrule))
                                            {

                                                continue;
                                            }
                                        }
                                    }
                                }

                            }


                            if (o.Stack < output.Triggers[0].RequiredCount)
                            {
                                continue;
                            }




                            r.PlaceInMachine(r.GetMachineData(), o, false, f);

                            d.Items.Reduce(o, output.Triggers[0].RequiredCount);
                            f.Items.Reduce(o, output.Triggers[0].RequiredCount);

                            return;
                        }
                        else if (r.heldObject.Value == null && r.MinutesUntilReady != 0)
                        {
                            r.MinutesUntilReady = 0;

                        }
                    }
                }
            }
            catch (Exception e)
            {

            }
        }
        static void transfer4(Chest DC, StardewValley.Object d, Chest r)  //for a object -->chest
        {
            Console.WriteLine("Transfer4" + DC.TileLocation);
            if (checkForSignal(DC) == false)
            {
                return;
            }




            foreach (StardewValley.Object DCo in DC.Items) // for each object that matches the object in DC chest, unless its a signal then we ignore this.
            {


                if (DCo != null && d.heldObject.Value != null && d.readyForHarvest.Value && r.Items.Count < r.GetActualCapacity() && (d.heldObject.Value.ItemId == DCo.ItemId || DC.Items.ContainsId("SIGNAL")))
                {

                    StardewValley.Object o = new StardewValley.Object();
                    d.heldObject.Value.DeepCloneTo(o);  //remove the item
                    d.heldObject.Value = null;

                    addStack(o.ItemId, r, o.Stack); // add it


                }
            }
        }

        static void transferB(StardewValley.Buildings.Building build, Chest chst)
        {
            if (build is StardewValley.Buildings.FishPond fish)
            {
                if (fish.FishCount > 2)
                {
                    fish.CatchFish();
                    chst.addItem(fish.GetFishObject());
                }
                if (fish.output.Value != null)
                {
                    Item o = fish.output.Value;
                    fish.output.Value.DeepCloneTo(o);
                    fish.output.Value = null;
                    chst.addItem(o);
                }
            }

        }

        public static void GetplacementAction2()
        {

            try
            {
                InitiateDirectionChests();
                InitiateBuildingChests();
            }
            catch (Exception e)
            {

                Console.WriteLine("Ah fuck. Something in the Harmony patch \"GetplacementAction2\" went wrong.");

            }
        }

        public static void GetperformRemoveAction()
        {
            try
            {
                InitiateDirectionChests();
                InitiateBuildingChests();
            }
            catch (Exception e)
            {

                Console.WriteLine("Ah fuck. Something in the Harmony patch \"GetperformRemoveAction\" went wrong.");

            }
        }
    }
}
